package com.minlinuxserver.servlet;

import java.io.IOException;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import com.minlinuxserver.dao.LinkDao;
import com.minlinuxserver.dao.impl.LinkDaoImpl;
import com.minlinuxserver.domain.Link;
import com.minlinuxserver.util.QueryStringBuilder;

/**
 * @author Peter Lindblom
 * 
 */
public class LinksAdminServlet extends BaseAdminServlet {

	private static final long serialVersionUID = 1L;
	private LinkDao linkDao = new LinkDaoImpl();

	private static final String PARAMETER_ACTION = "action";
	private static final String PARAMETER_ID = "id";
	private static final String PARAMETER_LINK_NAME = "linkName";
	private static final String PARAMETER_LINK_HREF = "linkHref";
	private static final String PARAMETER_LINK_SORT_INDEX = "linkSortIndex";

	private static final String ATTRIBUTE_LIST_OF_LINKS = "listOfLinks";
	private static final String ATTRIBUTE_SELECTED_LINK = "selectedLink";

	protected void doGet(HttpServletRequest req, HttpServletResponse resp)
			throws ServletException, IOException {
		super.doGet(req, resp);

		req.setAttribute(ATTRIBUTE_LIST_OF_LINKS, linkDao.getAllLinks());

		if (req.getParameter(PARAMETER_ACTION) == null) {
			forward(req, resp);
		} else if (req.getParameter(PARAMETER_ACTION).equals(
				ActionType.GET.toString())
				&& req.getParameter(PARAMETER_ID) != null) {
			req.setAttribute(ATTRIBUTE_SELECTED_LINK, linkDao
					.getLinkById(new Long(req.getParameter(PARAMETER_ID))
							.longValue()));
			forward(req, resp);
		}
	}

	@Override
	protected void doPost(HttpServletRequest req, HttpServletResponse resp)
			throws ServletException, IOException {
		super.doPost(req, resp);

		if (ActionType.ADD.isThisType(req.getParameter(PARAMETER_ACTION))) { // Lägg
																				// till
																				// Länk

			if (isValidLink(req.getParameter(PARAMETER_LINK_NAME), req
					.getParameter(PARAMETER_LINK_HREF), req
					.getParameter(PARAMETER_LINK_SORT_INDEX))) {
				Link link = new Link();

				link.setLinkName(req.getParameter(PARAMETER_LINK_NAME));
				link.setLinkHref(req.getParameter(PARAMETER_LINK_HREF));
				link.setLinkSortIndex(new Long(req
						.getParameter(PARAMETER_LINK_SORT_INDEX)));

				linkDao.saveLink(link);
			}

			resp.sendRedirect(getServletPath());

		} else if (ActionType.EDIT.isThisType(req
				.getParameter(PARAMETER_ACTION))) { // Uppdatera

			if (isValidLink(req.getParameter(PARAMETER_LINK_NAME), req
					.getParameter(PARAMETER_LINK_HREF), req
					.getParameter(PARAMETER_LINK_SORT_INDEX))) {
				Link link = linkDao.getLinkById(new Long(req
						.getParameter(PARAMETER_ID)).longValue());

				link.setLinkName(req.getParameter(PARAMETER_LINK_NAME));
				link.setLinkHref(req.getParameter(PARAMETER_LINK_HREF));
				link.setLinkSortIndex(new Long(req
						.getParameter(PARAMETER_LINK_SORT_INDEX)));

				linkDao.saveLink(link);
			}

			QueryStringBuilder queryStringBuilder = new QueryStringBuilder(getServletPath());
			queryStringBuilder.appendKeyAndValue(PARAMETER_ACTION, ActionType.GET.toString());
			queryStringBuilder.appendKeyAndValue(PARAMETER_ID, req.getParameter(PARAMETER_ID));
			
			resp.sendRedirect(queryStringBuilder.buildUrl());
			
		} else if (ActionType.DELETE.isThisType(req
				.getParameter(PARAMETER_ACTION))) {

			Link temp = linkDao.getLinkById(new Long(req
					.getParameter(PARAMETER_ID)).longValue());
			linkDao.deleteLink(temp);
			resp.sendRedirect(getServletPath());

		}
	}

	private boolean isValidLink(String linkName, String linkHref,
			String linkSortIndex) {
		if (linkName != null && linkName.length() != 0
				&& linkName.length() <= 500

				&& linkHref != null && linkHref.length() != 0
				&& linkHref.length() <= 1000

				&& linkSortIndex != null && linkSortIndex.length() != 0
				&& linkSortIndex.length() <= 10
				&& new Long(linkSortIndex).longValue() >= 0

		) {
			return true;
		}
		return false;
	}

}
