package com.minlinuxserver.servlet;

import java.io.IOException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import com.minlinuxserver.dao.MessageDao;
import com.minlinuxserver.dao.impl.MessageDaoImpl;
import com.minlinuxserver.domain.*;
import com.minlinuxserver.util.QueryStringBuilder;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * @author Peter Lindblom
 * 
 */
public class ShutDownMessageAdminServlet extends BaseAdminServlet {
	private static Logger logger = LoggerFactory
			.getLogger(ShutDownMessageAdminServlet.class);
	private static final long serialVersionUID = 1L;
	private SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
	private MessageDao messageDao = new MessageDaoImpl();;

	private static final String PARAMETER_ACTION = "action";
	private static final String PARAMETER_ID = "id";
	private static final String PARAMETER_MESSAGE = "message";
	private static final String PARAMETER_MESSAGE_TYPE = "message-type";
	private static final String PARAMETER_DATE = "date";
	private static final String PARAMETER_LAST_MESSAGE_ID = "last-message-id";

	private static final String ATTRIBUTE_LIST_OF_MESSAGES = "ListOfMessages";
	private static final String ATTRIBUTE_SELECTED_MESSAGE = "SelectedMessage";
	private static final String ATTRIBUTE_SELECTED_MESSAGE_TYPE = "SelectedMessageType";
	private static final String ATTRIBUTE_MESSAGE_TYPES = "MessageTypes";

	protected void doGet(HttpServletRequest req, HttpServletResponse resp)
			throws ServletException, IOException {

		super.doGet(req, resp);
		req.setAttribute(ATTRIBUTE_LIST_OF_MESSAGES, messageDao
				.getAllMessages());		
		req.setAttribute(ATTRIBUTE_MESSAGE_TYPES, MessageTypeHelper.getMessageTypesAsStrings());

		if (req.getParameter(PARAMETER_ACTION) == null) {
			forward(req, resp);
		} else if (ActionType.GET
				.isThisType(req.getParameter(PARAMETER_ACTION))
				&& req.getParameter(PARAMETER_ID) != null) {
			Message message = messageDao.getMessageById(new Long(req.getParameter(PARAMETER_ID)).longValue());
			req.setAttribute(ATTRIBUTE_SELECTED_MESSAGE, message);
			req.setAttribute(ATTRIBUTE_SELECTED_MESSAGE_TYPE, message.getMessageType());
			forward(req, resp);
		}
	}

	
	
	@Override
	protected void doPost(HttpServletRequest req, HttpServletResponse resp)
			throws ServletException, IOException {
		super.doPost(req, resp);

		if (ActionType.ADD.isThisType(req.getParameter(PARAMETER_ACTION))) { // Lägg
			// till
			// meddelande

			if (isValidMessage(req.getParameter(PARAMETER_MESSAGE), req
					.getParameter(PARAMETER_DATE), req.getParameter(PARAMETER_MESSAGE_TYPE))) {

				Date date = null;

				try {
					date = sdf.parse(req.getParameter(PARAMETER_DATE));
				} catch (ParseException e) {
					exceptionHandling(e);
				}

				
				Message newMessage = null;
				
				if (MessageType.UP_TIME_MESSAGE.isThisType(req
						.getParameter(PARAMETER_MESSAGE_TYPE))) {
					newMessage = MessageFactory.createUpTimeMessage();					
				} else if (MessageType.DOWN_TIME_MESSAGE.isThisType(req
						.getParameter(PARAMETER_MESSAGE_TYPE))) {
					newMessage = MessageFactory.createDownTimeMessage();
				}
				
				newMessage.setDate(date);
				newMessage.setMessage(req
						.getParameter(PARAMETER_MESSAGE));
				newMessage.setPreviousMessage(getMessageWithId(req
						.getParameter(PARAMETER_LAST_MESSAGE_ID)));
				
				
				messageDao.saveMessage(newMessage);

			}

			resp.sendRedirect(getServletPath());

		} else if (ActionType.EDIT.isThisType(req
				.getParameter(PARAMETER_ACTION))) { // Updatera

			if (isValidMessage(req.getParameter(PARAMETER_MESSAGE), req
					.getParameter(PARAMETER_DATE), req.getParameter(PARAMETER_MESSAGE_TYPE))) {
				Message mess = messageDao.getMessageById(new Long(req
						.getParameter(PARAMETER_ID)).longValue());

				mess.setMessageType(req.getParameter(PARAMETER_MESSAGE_TYPE));
				mess.setMessage(req.getParameter(PARAMETER_MESSAGE));
				try {
					mess.setDate(sdf.parse(req.getParameter(PARAMETER_DATE)));
				} catch (ParseException e) {
					exceptionHandling(e);
				}

				mess.setPreviousMessage(getMessageWithId(req
						.getParameter(PARAMETER_LAST_MESSAGE_ID)));

				messageDao.saveMessage(mess);
			}

			QueryStringBuilder queryStringBuilder = new QueryStringBuilder(
					getServletPath());
			queryStringBuilder.appendKeyAndValue(PARAMETER_ACTION,
					ActionType.GET.toString());
			queryStringBuilder.appendKeyAndValue(PARAMETER_ID, req
					.getParameter(PARAMETER_ID));

			resp.sendRedirect(queryStringBuilder.buildUrl());

		} else if (ActionType.DELETE.isThisType(req
				.getParameter(PARAMETER_ACTION))) {

			Message temp = messageDao.getMessageById(new Long(req
					.getParameter(PARAMETER_ID)).longValue());
			messageDao.deleteMessage(temp);
			resp.sendRedirect(getServletPath());
		}
	}

	private void exceptionHandling(Exception exception) {
		logger.error(exception.getMessage());
	}

	private boolean isValidMessage(String message, String date, String messageType) {
		if (message != null && message.length() != 0 && message.length() <= 500
				&& date != null && date.length() != 0 && date.length() <= 10 
				&& MessageTypeHelper.isValidMessageType(messageType))
		{
			return true;
		}
		return false;
	}

	private Message getMessageWithId(String id) {

		if (id != null && id.length() > 0) {
			return messageDao.getMessageById(new Long(id).longValue());
		}

		return null;
	}

}
