package com.minlinuxserver.servlet;

import java.io.IOException;
import java.util.List;
import javax.servlet.*;
import javax.servlet.http.*;
import com.minlinuxserver.domain.Message;
import com.minlinuxserver.dao.MessageDao;
import com.minlinuxserver.dao.impl.MessageDaoImpl;

/**
 * @author Peter Lindblom
 * 
 */
public class ShutDownMessageServlet extends BaseServlet {

	private static final long serialVersionUID = 1L;

	private static final String PARAMETER_GET = "get";
	private static final String ATTRIBUTE_RESULT_LIST = "resultList";

	private enum listType {
		ALL("all"), LATEST("latest");

		private String stringValue;

		private listType(String stringValue) {
			this.stringValue = stringValue;
		}

		public boolean isThisListType(String stringValue) {
			if (this.stringValue != null) {
				return this.stringValue.equalsIgnoreCase(stringValue);
			}
			return false;
		}

		@Override
		public String toString() {
			return stringValue;
		}
	};

	@Override
	protected void doGet(HttpServletRequest req, HttpServletResponse resp)
			throws ServletException, IOException {

		if (listType.LATEST.isThisListType(req.getParameter(PARAMETER_GET))) {

			req.setAttribute(ATTRIBUTE_RESULT_LIST,
					getLatestMessages(listType.LATEST));
			forward(req, resp);

		} else if (listType.ALL.isThisListType(req.getParameter(PARAMETER_GET))) {

			req.setAttribute(ATTRIBUTE_RESULT_LIST,
					getLatestMessages(listType.ALL));
			forward(req, resp);
		}
	}

	private List<Message> getLatestMessages(listType typeOfListing) {

		MessageDao messageDao = new MessageDaoImpl();
		if (typeOfListing.equals(listType.ALL)) {
			return messageDao.getAllMessages();
		} else if (typeOfListing.equals(listType.LATEST)) {
			return messageDao.getLatestMessage(1);
		} else {
			return messageDao.getLatestMessage(1);
		}
	}
}
