package com.minlinuxserver.util;


import java.util.Iterator;
import java.util.Map;
import java.util.SortedMap;
import java.util.TreeMap;
import java.util.Map.Entry;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;

public class QueryStringBuilder {
	private String url = "";
	
	private SortedMap<String, String[]> keyValuePairs = new TreeMap<String, String[]>(); 
	
	public QueryStringBuilder() {
	}

	public QueryStringBuilder(String url) {
		this.url = url;
	}
	
	public QueryStringBuilder appendKeyAndValue(String key, String value){		
		keyValuePairs.put(key, new String[] {value});		
		return this;
	}
	
	public QueryStringBuilder appendKeyAndValue(String key, String[] values){		
		keyValuePairs.put(key, values);		
		return this;
	}
	
	public QueryStringBuilder appendMap(Map<String, String[]> map) {
		keyValuePairs.putAll(map);
		return this;
	}
	
	public String toString(){	
		return buildUrl();
	}

	public String buildUrl(){
		StringBuilder stringBuilder = new StringBuilder(url);

		if (keyValuePairs.size() > 0) {
			stringBuilder.append("?");
			for (Iterator<Entry<String, String[]>> it = keyValuePairs.entrySet()
					.iterator(); it.hasNext();) {
				Entry<String, String[]> currentEntry = it.next();
				
				String[] values = currentEntry.getValue();
				
				for(int i=0;i < values.length; i++ ){
					stringBuilder.append(urlEncode(currentEntry.getKey()));
					stringBuilder.append("=");
					stringBuilder.append(urlEncode(values[i]));
					
					if(i < (values.length -1) ){
						stringBuilder.append("&");
					}
				}
								
				if (it.hasNext()) {
					stringBuilder.append("&");
				}
			}
		}

		return stringBuilder.toString();
	}
	
	private String urlEncode(String input){
		try {
			return URLEncoder.encode(input,"UTF-8");
		} catch (UnsupportedEncodingException e) {
			return "";
		}
	}
	
}
