package com.minlinuxserver.dao.impl;

import com.minlinuxserver.domain.MessageType;
import java.util.Date;
import java.util.List;

import com.minlinuxserver.dao.*;
import com.minlinuxserver.domain.*;
import junit.framework.TestCase;

public class TestMessageDaoImpl extends TestCase {

	MessageDao messageDao = new MessageDaoImpl();
	
	public void testSaveMessage() throws Exception {
		
		Message message = MessageFactory.createUpTimeMessage();
		
		message.setMessage("up 1 2 3");
		message.setDate(new Date());
		message.setPreviousMessage(null);
		
		assertEquals(true, messageDao.saveMessage(message));		
	}
	
	public void testMessageTypes() throws Exception {
		
		Message message = MessageFactory.createUpTimeMessage();
		
		message.setMessage("up");
		message.setDate(new Date());
		message.setPreviousMessage(null);
		
		assertEquals(true, messageDao.saveMessage(message));
		
		message = MessageFactory.createDownTimeMessage();
		
		message.setMessage("down");
		message.setDate(new Date());
		message.setPreviousMessage(null);
		
		assertEquals(true, messageDao.saveMessage(message));
		
		List<Message> messages = messageDao.getAllMessages(); 
		
		int upTimeCount = 0;
		int downTimeCount = 0;
		
		for(Message tmpMessage : messages){
			if(MessageType.DOWN_TIME_MESSAGE.isThisType(tmpMessage.getMessageType())){
				assertNotNull(tmpMessage.getId());
				assertNotNull(tmpMessage.getMessage());
				assertNotNull(tmpMessage.getDate());
				downTimeCount++;
			}
			if(MessageType.UP_TIME_MESSAGE.isThisType(tmpMessage.getMessageType())){
				assertNotNull(tmpMessage.getId());
				assertNotNull(tmpMessage.getMessage());
				assertNotNull(tmpMessage.getDate());
				upTimeCount++;
			}
		}
		
		assertEquals(true, (upTimeCount > 0) );
		assertEquals(true, (downTimeCount > 0) );
		
		
	}
	
	public void testGetAllMessages() throws Exception {
		
		List<Message> messages = messageDao.getAllMessages(); 
		
		int count = 0;
		
		for(Message message : messages){
			if(message instanceof Message){
				assertNotNull(message.getId());
				assertNotNull(message.getMessage());
				assertNotNull(message.getDate());
				count++;
			}
		}
		
		assertEquals(true, (count > 0) );
		
	}
	
	public void testGetLatestMessage() throws Exception {
		
		List<Message> messages = messageDao.getLatestMessage(1); 
		
		int count = 0;
		
		for(Message message : messages){
			if(message instanceof Message){
				assertNotNull(message.getId());
				assertNotNull(message.getMessage());
				assertNotNull(message.getDate());
				count++;
			}
		}
		
		assertEquals(true, (count == 1) );		
	}
	
	public void testGetMessageById() throws Exception {
		
		List<Message> messages = messageDao.getLatestMessage(1); 
		
		long latestIdIs = 0;
		
		for(Message message : messages){
			if(message instanceof Message){
				latestIdIs = message.getId();				
			}
		}
		
		Message tmpMessage = messageDao.getMessageById(latestIdIs);
		
		
		assertEquals(true, (tmpMessage instanceof Message) );		
	}
	
	public void testDeleteMessage() throws Exception {
	
		int numberOfMessageBefore = messageDao.getAllMessages().size(); 
		
		
		assertEquals(true, messageDao.deleteMessage(messageDao.getMessageById(getLastId())) );
		
		int numberOfMessageAfter = messageDao.getAllMessages().size();
		
		
		
		assertEquals((numberOfMessageBefore-1), numberOfMessageAfter);
	}
	
	public void testChangeMessageType() throws Exception {
		
		Date testDate = new Date();
		String testMessageString = "1234567890";
		Long testid;
		
		Message message = MessageFactory.createDownTimeMessage();		
		message.setDate(testDate);
		message.setMessage(testMessageString);
		
		messageDao.saveMessage(message);
		
		Message message2 = messageDao.getTheLatestMessage();
		testid = message2.getId();
		
		Message message3 = message2;
		message3.setMessageType(MessageType.UP_TIME_MESSAGE.toString());
		
		messageDao.saveMessage(message3);
		
		assertEquals(true, MessageType.UP_TIME_MESSAGE.isThisType(message3.getMessageType()));
		
		Message message4 = messageDao.getTheLatestMessage();
		
		assertEquals(testid.longValue(), message4.getId().longValue());
		
		
	}
	
	private long getLastId(){
		long lastId = 0;
		List<Message> messages = messageDao.getLatestMessage(1);
		
		for(Message message : messages){
			if(message instanceof Message){
				lastId = message.getId();				
			}
		}
		return lastId;
	}
	
}
